#!/usr/bin/env python
#coding=utf-8

import onelab, sys

# we work in the directory where the script is located
workdir = onelab.path(__file__) 

OL = onelab.client()

EPIDERMIS = OL.defineNumber('Parameters/Skin/2EPIDERMIS', value=0.05, 
        label='Thickness of epidermis [mm]', readOnly=1)
DERMIS = OL.defineNumber('Parameters/Skin/3DERMIS', value=1.5, 
        label='Thickness of dermis [mm]')

SKINTYPE = OL.defineNumber('Parameters/Skin/1SKINTYPE', value=1, 
        label='Skin type', labels = {1:'hairy (thin)', 2:'glabrous (thick)'})
if SKINTYPE == 1 :
    EPIDERMIS = 0.05
else :
    EPIDERMIS = 0.12
OL.setNumber('Parameters/Skin/2EPIDERMIS', value=EPIDERMIS)

# Display the geometry in the GUI
OL.openProject(onelab.path(__file__, 'laser.geo'))

# Check for Onelab parameters in the .geo file
# Gmsh is called with OL.run because it is native
OL.run('mesher', 'gmsh - ' +  onelab.path(__file__, 'laser.geo'),'')


REFLECTIVITY = OL.defineNumber('Parameters/Skin/4REFLECTIVITY', value=0.0078, 
        label='Skin reflectivity []')
TCORE = OL.defineNumber('Parameters/Skin/5TCORE', value=37, 
        label='Core temperature [C]')
TBASE = OL.defineNumber('Parameters/Skin/6TBASE', value=30, 
        label='Baseline temperature [C]')

# Flags describing model features that are activated or not
TENEUR = OL.defineNumber('Parameters/Skin/8TENEUR', value=1, choices = [0, 1],
                          label='Account for variable water content')
CONVBC = OL.defineNumber('Parameters/Skin/9CONVBC', value=0, choices = [0, 1],
                          label='Account for convection')

WCONTENT = OL.defineNumber('Parameters/Skin/WCONTENT', value=0.65, 
        label='Water content []', visible= not TENEUR)

HCONV = OL.defineNumber('Parameters/Skin/HCONV', value=100, 
        label='Convection coefficient [W/(Km)]', visible=CONVBC)
TAMBIANT = OL.defineNumber('Parameters/Skin/TAMBIANT', value=20, 
        label='Ambiant temperature [C]', visible=CONVBC)

# Available LASER models
LASERSHAPE = OL.defineNumber('Parameters/Laser/1LASERSHAPE', value=2, 
        label='Laser shape', labels = {1:'Gaussian', 2:'Flat-top'})
ABSORPTION = OL.defineNumber('Parameters/Laser/3ABSORPTION', value=2e4, 
        label='Absorption coefficient [1/m]')

LASERSIMU = OL.defineNumber('Parameters/Laser/4LASERSIMU', value=1, 
        label='Laser type',
        labels = {1:'Imposed power', 2:'Imposed power file',
                  3:'Controlled temperature', 4:'Controlled temperature file'})
LASERTYPE = OL.defineNumber('Parameters/Laser/LASERTYPE', value=1, 
        label='Laser category', visible=0)

if LASERSIMU==1 or LASERSIMU==2 :
    OL.setNumber('Parameters/Laser/LASERTYPE', value=1)
else :
    OL.setNumber('Parameters/Laser/LASERTYPE', value=2)


LASERPOWER = OL.defineNumber('Parameters/Laser/5LASERPOWER', value=10.0, 
        label='Power [W]')
LASERTEMP = OL.defineNumber('Parameters/Laser/5LASERTEMP', value=50, 
        label='Target temperature T_hot [C]')
STIMTIME = OL.defineNumber('Parameters/Laser/6STIMTIME', value=0.10, 
        label='Pulse Duration [s]')
TEMP1 = OL.defineNumber('Parameters/Laser/7TEMP1', value=30, 
        label='Conditioning temperature T_cond [C]')
TIME1 = OL.defineNumber('Parameters/Laser/8TIME1', value=0, 
        label='Conditioning time [s]')
RAMPTIME = OL.defineNumber('Parameters/Laser/9RAMPTIME', value=0.01, 
        label='Ramp up time [s]')

TimeStep = 1e-3 * OL.defineNumber('Parameters/Elmer/2TimeStep', value=3, 
        label='Time step [ms]')
TimeEnd = OL.defineNumber('Parameters/Elmer/3TimeEnd', value=0.25, 
        label='Simulation end time [s]')

TSKINFILE = OL.defineString('Parameters/Laser/6TSKINFILE', value='', 
        label='Imposed temp. file', choices=['Timposed.ol','TMoureaux.ol'])
QSKINFILE = OL.defineString('Parameters/Laser/6QSKINFILE', value='', 
        label='Imposed power file', choices=['pin.ol'])

if LASERSIMU == 1 :
    OL.setNumber('Parameters/Laser/5LASERTEMP', visible=0)
    OL.setNumber('Parameters/Laser/5LASERPOWER', visible=1)
    OL.setNumber('Parameters/Laser/6STIMTIME', visible=1)
    OL.setNumber('Parameters/Laser/7TEMP1', visible=0)
    OL.setNumber('Parameters/Laser/8TIME1', visible=0)
    OL.setNumber('Parameters/Laser/9RAMPTIME', visible=0)
    OL.setString('Parameters/Laser/6TSKINFILE', visible=0)
    OL.setString('Parameters/Laser/6QSKINFILE', visible=0)

if LASERSIMU == 2 :
    OL.setNumber('Parameters/Laser/5LASERTEMP', visible=0)
    OL.setNumber('Parameters/Laser/5LASERPOWER', visible=0)
    OL.setNumber('Parameters/Laser/6STIMTIME', visible=0)
    OL.setNumber('Parameters/Laser/7TEMP1', visible=0)
    OL.setNumber('Parameters/Laser/8TIME1', visible=0)
    OL.setNumber('Parameters/Laser/9RAMPTIME', visible=0)
    OL.setString('Parameters/Laser/6TSKINFILE', visible=0)
    OL.setString('Parameters/Laser/6QSKINFILE', visible=1)

if LASERSIMU == 3 :
    OL.setNumber('Parameters/Laser/5LASERTEMP', visible=1)
    OL.setNumber('Parameters/Laser/5LASERPOWER', visible=1)
    OL.setNumber('Parameters/Laser/6STIMTIME', visible=1)
    OL.setNumber('Parameters/Laser/7TEMP1', visible=1)
    OL.setNumber('Parameters/Laser/8TIME1', visible=1)
    OL.setNumber('Parameters/Laser/9RAMPTIME', visible=1)
    OL.setString('Parameters/Laser/6TSKINFILE', visible=0)
    OL.setString('Parameters/Laser/6QSKINFILE', visible=0)
    
if LASERSIMU == 4 :
    OL.setNumber('Parameters/Laser/5LASERTEMP', visible=0)
    OL.setNumber('Parameters/Laser/5LASERPOWER', visible=1)
    OL.setNumber('Parameters/Laser/6STIMTIME', visible=0)
    OL.setNumber('Parameters/Laser/7TEMP1', visible=0)
    OL.setNumber('Parameters/Laser/8TIME1', visible=0)
    OL.setNumber('Parameters/Laser/9RAMPTIME', visible=0)
    OL.setString('Parameters/Laser/6TSKINFILE', visible=1)
    OL.setString('Parameters/Laser/6QSKINFILE', visible=0)

# Guess the pulse end time
if LASERSIMU == 1 :
    PULSEEND = STIMTIME
elif LASERSIMU == 2 :
    PULSEEND = 0
elif LASERSIMU == 3 :
    PULSEEND = TIME1 + RAMPTIME + STIMTIME
elif LASERSIMU == 4 :
    PULSEEND = 0
    
# Parametrers of the post-processing
PROBEDEPTH = OL.defineNumber('PostPro/1PROBEDEPTH', value=EPIDERMIS/2.,
                        label='Probe depth [mm]')
if PROBEDEPTH > EPIDERMIS + DERMIS or PROBEDEPTH < 0:
    OL.setNumber('PostPro/1PROBEDEPTH', value=EPIDERMIS/2.)
    
PROBETIME = OL.defineNumber('PostPro/2PROBETIME', value=PULSEEND,
                            label='Probe time [s]')
if PROBETIME > TimeEnd-TimeStep or PROBETIME < 0:
    OL.setNumber('PostPro/2PROBETIME', value=TimeEnd-TimeStep)

OVERTEMP = OL.defineNumber('PostPro/5OVERTEMP', value=39.5,
                           label='Thermal threshold fiber [C]')

# Highlight post-processing parmeters that need be set by the user
# and suggest appropriate values with the "range" widget
EPS=1e-3 #mm
OL.setNumber('PostPro/1PROBEDEPTH', attributes={'Highlight':'AliceBlue'},
             choices=[ EPIDERMIS/2.])
OL.setNumber('PostPro/2PROBETIME', attributes={'Highlight':'AliceBlue'},
             choices=[ PULSEEND ])
OL.setNumber('PostPro/5OVERTEMP', attributes={'Highlight':'AliceBlue'},
             choices=[ 39.5 ] )

# internal variables used by the clients
OL.setNumber('PostPro/3SKINWIDTH', value=DERMIS+EPIDERMIS, visible=0,
    label='Skin width [mm]')
OL.setNumber('PostPro/9ZSURF', label='z values [mm]',
             value=DERMIS+EPIDERMIS-PROBEDEPTH, visible=0,
             choices=[ DERMIS+EPIDERMIS-EPS, DERMIS-EPS, DERMIS+EPIDERMIS-PROBEDEPTH] )

# Preprocess .ol files
OL.preProcess(onelab.path(__file__, 'laser.sif.ol'))
OL.preProcess(onelab.path(__file__, 'script.opt.ol'))
OL.preProcess(onelab.path(__file__, 'script0.opt.ol'))
OL.preProcess(onelab.path(__file__, 'script2.opt.ol'))

solFiles = OL.solutionFiles( [onelab.path(__file__, 'solution.pos')] )

# Files saved when the "Archive output files automatically" option is enabled
OL.outputFiles([onelab.path(__file__, 'laser.pdf')])


print solFiles

# we're done with the "check" phase
if OL.action == 'check' :
    exit(1)

# Mesh the geometry
OL.run('mesher', 'gmsh - ' +  onelab.path(__file__, 'laser.geo'),'-3')

# Display the mesh in the GUI
OL.mergeFile(onelab.path(__file__, 'laser.msh'))

# Call Elmer clients
OL.call('ElmerGrid 14 2 laser.msh -out mesh', rundir=workdir)

# Call Elmer and display the field solution in the GUI
if not OL.restoreSolution() :
    OL.call('ElmerSolver laser.sif', rundir=workdir)

OL.mergeFile(onelab.path(__file__, solFiles[0]))

# Call postprocessing clients

OL.call('gmsh ' + solFiles[0] + ' script0.opt -', rundir=workdir)

# Extract values from metamodel output files and upload them in the GUI
MAXTEMPTIME = onelab.extract(onelab.path(__file__, 'tempMaxInterface.txt'), -1, 2)
TEMPINTERF = onelab.extract(onelab.path(__file__, 'tempMaxInterface.txt'), -1, 8)
UADELTA = onelab.extract(onelab.path(__file__, 'Adelta.txt'), -1, 8)
OL.setNumber('Solution/MAXTEMPTIME', value=MAXTEMPTIME, readOnly=1, 
                attributes={'Highlight':'Coral'},
                label='Max temp. time at interface [s]')
OL.setNumber('Solution/TEMPINTERF', value=TEMPINTERF, readOnly=1, 
                attributes={'Highlight':'Coral'},
                label='Max temperature at interface [C]')
OL.setNumber('Solution/UADELTA', value=UADELTA, readOnly=1, 
             attributes={'Highlight':'Coral'},
             label='Area A.T. at interface [mm2]')


OL.call('gmsh ' + solFiles[0] + ' script.opt -', rundir=workdir)

OL.call('gmsh ' + solFiles[0] + ' script2.opt -', rundir=workdir)
VOLUME = onelab.extract(onelab.path(__file__, 'volmax.txt'), -1, 8)
VOLDERM = onelab.extract(onelab.path(__file__, 'voldermmax.txt'), -1, 8)
OL.defineNumber('Solution/VOLUME', readOnly=1, 
                attributes={'Highlight':'Coral'},
                label='Skin volume A.T. [mm3]')
OL.defineNumber('Solution/VOLDERM', readOnly=1, 
                attributes={'Highlight':'Coral'},
                label='Derm volume A.T. [mm3]')
OL.addNumberChoice('Solution/VOLUME', VOLUME)
OL.addNumberChoice('Solution/VOLDERM', VOLDERM)

# Call gnuplot client
OL.preProcess(onelab.path(__file__, 'plot.plt.ol'))
OL.preProcess(onelab.path(__file__, 'plot1.plt.ol'))
OL.preProcess(onelab.path(__file__, 'plot2.plt.ol'))
OL.preProcess(onelab.path(__file__, 'plot3.plt.ol'))
OL.preProcess(onelab.path(__file__, 'plot4.plt.ol'))

OL.call('gnuplot plot.plt', rundir=workdir)

if OL.batch:
    sys.stderr.write('%s: MAXTEMPTIME = %12.5e (%12.5e)\n' %
                     (__file__,MAXTEMPTIME, 9.90000e-02))
    sys.stderr.write('%s: UADELTA = %12.5e (%12.5e)\n' %
                     (__file__,UADELTA, 2.91434e+01))



